#!/bin/bash
#
# X-tee MISP2 portaalide andmestruktuur PostgreSQL
#
# Aktors 2015
# 16.02.2015 versioon (postgresql-9.3)

xrd_prefix=/usr/xtee

workdir=/root/xteesrc
pgsql_default_port=5432
pgsql_dir=/usr/lib/postgresql/9.3/bin
current_version="2.1.6"
dpkg -s "xtee-misp2-postgresql" > /tmp/xtee-postgresql.version
version=$(awk -F ": " '/Config-Version/ { print $2 }' /tmp/xtee-postgresql.version)
echo "You have package version " $version;
mkdir -p $workdir

cd $xrd_prefix/db/sql
echo
if [ ! -f $pgsql_dir/psql ] ;
then
	echo "PostgreSQL 9.3 should be installed and running"
	echo -n "Is PostgreSQL 9.3 installed and running (y/n)? [y] "
	read pgexists < /dev/tty
	if [ "$pgexists" == "" ] ;
	then
		pgexists="y"
	fi
	if [ `echo $pgexists | grep -i n ` ];
	then
		echo "Install and run PostgreSQL 9.3"
		echo
		exit 1
	fi

	echo -n "Please provide PostgreSQL client working directory [$pgsql_dir]? "
	read user_pgsql < /dev/tty
	if [ "$user_pgsql" == "" ];
	then
		user_pgsql=$pgsql_dir
	fi
	pgsql_dir=$user_pgsql
	if [ ! -f $pgsql_dir/psql ];
	then
		echo "$pgsql_dir/psql is not found"
		exit 1
	fi
fi


### port
echo -n "Please provide PostgreSQL server port [$pgsql_default_port]? "
read pgport < /dev/tty
if [ "$pgport" == "" ];
then
	pgport="$pgsql_default_port"
fi

### database
echo -n "Please provide database name that will be used: [misp2db] "
read dbname < /dev/tty
if [ "$dbname" == "" ];
then
	dbname="misp2db"
fi

### username
echo -n "Provide username that will be (or is)  host of database: [misp2] "
read username < /dev/tty
if [ "$username" == "" ];
then
	username="misp2"
fi


### schema name is the same as username
schema_name="$username"


### adding database
db_exists=upgrade
if $pgsql_dir/psql -p $pgport -U postgres -lqt | cut -d \| -f 1 | grep -w $dbname; then
	echo "Database $dbname was found. Upgrading."
    db_exists=upgrade
else
	echo "No database $dbname was found. Creating new."
    db_exists=add
fi



if [ `echo $db_exists | grep -i add` ] 
then
	### Creating database confirmation
	echo -n "Are you sure you want to create new database '$dbname' (y/n)? [y]"
	read confirm_new_db_creation < /dev/tty
	if [ "$confirm_new_db_creation" == "" ] ;
	then
		confirm_new_db_creation="y"
	fi
	
	if echo $confirm_new_db_creation | grep -i y ;
	then
		echo "Creating database '$dbname'"
	else
		echo "Did not create new database nor updated existing one. Run installation again with correct DB name."
		exit 1;
	fi

	$pgsql_dir/createdb -p $pgport $dbname -U postgres -T template0 -E UNICODE
	if [ ! "$PIPESTATUS" = "0" ];
	then
		echo "Cannot create database '$dbname'"
		echo "You can try to create \"$dbname\" by yourself: "
		echo "$pgsql_dir/createdb  -p $pgport $dbname -U postgres -E UNICODE"
		echo ""
		echo "When database is created please press Enter"
		read < /dev/tty
	fi
	
	### adding username 
	echo -n "Does user \"$username\" already exist in database? (y/n) [n] ? "
	read user_exists < /dev/tty
	if [ "$user_exists" == "" ];
	then
		user_exists=n
	fi
	if [ `echo $user_exists | grep -i n ` ]
	then
		echo "Adding new user $username"
		$pgsql_dir/createuser  -p $pgport -U postgres -P -A -D $username
		if [ ! "$PIPESTATUS" = "0" ];
		then
			echo "Cannot add new user '$username'"
			echo "If user does not exist in the database then create him by running: "
			echo "$pgsql_dir/createuser -p $pgport -U postgres -P -A -D $username"
			echo
			echo "When user is created please press Enter"
			read < /dev/tty
		fi
	fi

	echo "Updating database structure"
	schema_name=$username
	sed "s/<misp2_schema>/$schema_name/g" create_misp2_db.sql > $workdir/tmp.create_misp2_db.sql
	$pgsql_dir/psql  -p $pgport $dbname -U postgres -f $workdir/tmp.create_misp2_db.sql 
	$pgsql_dir/psql  -p $pgport $dbname -U postgres -c "grant all on schema $schema_name to $username;"
	sed "s/misp2/$schema_name/g" grant_misp2_db.sql > $workdir/tmp.grant_misp2_db.sql
	$pgsql_dir/psql  -p $pgport $dbname -U postgres -f $workdir/tmp.grant_misp2_db.sql

	if [ ! "$PIPESTATUS" = "0" ];
	then
		echo "Cannot create database structure"
		exit 1
	else
	echo "Database structure is created"
	fi
	# default classifiers upload
	echo -n "Load default classifiers (y/n)? [y] "
	read load < /dev/tty
	if [ "$load" == "" ];
	then
		load="y"
	fi

	if [ `echo $load | grep -i y ` ];
	then
		echo "Loading classifiers with user 'postgres'..."
		sed "s/misp2/$schema_name/g" classifier_dump.sql > $workdir/tmp.classifier_dump.sql
		$pgsql_dir/psql -p $pgport $dbname -U postgres -f $workdir/tmp.classifier_dump.sql -q
		if [ ! "$PIPESTATUS" = "0" ];
		then
			echo "Cannot load classifiers"
			exit 1
		fi
	fi

###  default xsl upload
	echo "Loading stylesheets with user 'postgres'..."
	sed "s/misp2/$schema_name/g" insert_xslt.sql > $workdir/tmp.insert_xslt.sql
	$pgsql_dir/psql -p $pgport $dbname -U postgres -f $workdir/tmp.insert_xslt.sql -q
	if [ ! "$PIPESTATUS" = "0" ];
	then
		echo "Cannot load stylesheets"
		exit 1
	fi
	echo "Finished creating new database structure"
fi


### database already exists ###
if [ `echo $db_exists | grep -i upgrade` ]
then
	rm $workdir/tmp.alter* 
	sed "s/misp2\./$schema_name\./g" alter_table_1.0.50.sql  > $workdir/tmp.alter_1.0.50.sql
	sed "s/misp2\./$schema_name\./g" alter_table_1.0.51.sql  > $workdir/tmp.alter_1.0.51.sql
	sed "s/misp2\./$schema_name\./g" alter_table_2.1.0.sql  > $workdir/tmp.alter_2.1.0.sql
	sed "s/misp2\./$schema_name\./g" alter_table_2.1.1.sql  > $workdir/tmp.alter_2.1.1.sql
	sed "s/misp2\./$schema_name\./g" alter_table_2.1.2.sql  > $workdir/tmp.alter_2.1.2.sql
	sed "s/misp2\./$schema_name\./g" alter_table_2.1.3.sql  > $workdir/tmp.alter_2.1.3.sql
	sed "s/misp2\./$schema_name\./g" alter_table_2.1.4.sql  > $workdir/tmp.alter_2.1.4.sql

	touch $workdir/tmp.alter.sql
	if [ "$version" = "$current_version" ]
	then
               	echo "You have latest database version"
                version=1
	fi
	
	if [ "$version" = "2.0.0" ]
	then
		cat $workdir/tmp.alter_1.0.50.sql >> $workdir/tmp.alter.sql
		cat $workdir/tmp.alter_1.0.51.sql >> $workdir/tmp.alter.sql
		version="2.0.1"
	fi
	
	if [ "$version" = "2.0.1" ]
	then
		cat $workdir/tmp.alter_2.1.0.sql >> $workdir/tmp.alter.sql
		version="2.1.0"
	fi
	
	if [ "$version" = "2.1.0" ]
	then
		cat $workdir/tmp.alter_2.1.1.sql >> $workdir/tmp.alter.sql
		version="2.1.1"
	fi
	
	if [ "$version" = "2.1.1" ]
	then
		cat $workdir/tmp.alter_2.1.2.sql >> $workdir/tmp.alter.sql
		version="2.1.2"
	fi
	
	if [ "$version" = "2.1.2" ]
	then
		cat $workdir/tmp.alter_2.1.3.sql >> $workdir/tmp.alter.sql
		version="2.1.3"
	fi
	
	if [ "$version" = "2.1.3" ]
	then
		cat $workdir/tmp.alter_2.1.4.sql >> $workdir/tmp.alter.sql
		version="2.1.4"
	fi

	#if [ "$version" = "2.1.4" -o "$version" = "2.1.5" -o "$version" = "2.1.6"  ]
	#then
		#version="2.1.7"
	#fi
	
	if [ ! "$version" = "1" ]
        then
	        echo ""
		echo "Updating database structure... "
		$pgsql_dir/psql  -p $pgport $dbname -U postgres -f $workdir/tmp.alter.sql
		if [ ! "$PIPESTATUS" = "0" ];
		then
			echo "Failed updating structure. You can run the alter script by yourself running this line: psql -p $pgport $dbname -U postgres -f $workdir/tmp.alter.sql"
			exit 1
		fi
        fi
	echo "Updating classifiers with user 'postgres'..."
	sed "s/misp2/$schema_name/g" classifier_dump.sql > $workdir/tmp.classifier_dump.sql
	$pgsql_dir/psql -p $pgport $dbname -U postgres -f $workdir/tmp.classifier_dump.sql -q
	if [ ! "$PIPESTATUS" = "0" ];
	then
		echo "Failed updating system classifiers"
		exit 1
	fi
	echo "Updating stylesheets with user 'postgres'..."
	sed "s/misp2/$schema_name/g" insert_xslt.sql > $workdir/tmp.insert_xslt.sql
	$pgsql_dir/psql -p $pgport $dbname -U postgres -f $workdir/tmp.insert_xslt.sql -q
	if [ ! "$PIPESTATUS" = "0" ];
	then
		echo "Failed updating system XSL stylesheets"
		exit 1
	fi

	echo "Finished updating database structure"
fi
echo "";
