-- Function: misp2.where_to_send(text)

-- DROP FUNCTION misp2.where_to_send(text);

-- Create language plpgsql if it does not exist
CREATE OR REPLACE FUNCTION create_language_plpgsql()
RETURNS BOOLEAN AS $$
    CREATE LANGUAGE plpgsql;
    SELECT TRUE;
$$ LANGUAGE SQL;

SELECT CASE WHEN NOT
    (
        SELECT  TRUE AS exists
        FROM    pg_language
        WHERE   lanname = 'plpgsql'
        UNION
        SELECT  FALSE AS exists
        ORDER BY exists DESC
        LIMIT 1
    )
THEN
    create_language_plpgsql()
ELSE
    FALSE
END AS plpgsql_created;

DROP FUNCTION create_language_plpgsql();

CREATE OR REPLACE FUNCTION misp2.where_to_send(paring text)
  RETURNS text AS
$BODY$SELECT
  CASE WHEN q.type = 1
    THEN p.bpel_engine || '/' || split_part($1, '.', 1) || '/' || split_part($1, '.', 2)
    ELSE p.security_host || '/cgi-bin/consumer_proxy'
  END
FROM misp2.portal p
  LEFT JOIN misp2.query q ON (q.name = $1 OR
    q.name = split_part($1, '.', 2) || '.' || split_part($1, '.', 3))$BODY$
  LANGUAGE 'sql' VOLATILE
  COST 100;
ALTER FUNCTION misp2.where_to_send(text) OWNER TO misp2;

CREATE OR REPLACE FUNCTION misp2.where_to_send_test(portal_name text)
  RETURNS TABLE(sec_host text, portal_id integer) AS
$BODY$SELECT
p.security_host||'/cgi-bin/consumer_proxy', p.id
FROM misp2.portal p where p.short_name=$1$BODY$
  LANGUAGE 'sql' VOLATILE
  COST 100;
ALTER FUNCTION misp2.where_to_send_test(text) OWNER TO misp2;


-- Function: misp2.check_portal(text, text, text)
DROP TYPE IF EXISTS misp2.ret_type1 CASCADE;
CREATE TYPE misp2.ret_type1 AS (
	portal_id integer, 
	error_code integer, 
	error_message varchar(512), 
	client_xroad_instance varchar(64), 
	client_member_class varchar(16), 
	client_member_code varchar(20), 
	client_subsystem_code varchar(64));

DROP FUNCTION IF EXISTS misp2.check_portal(text, text) CASCADE;
-- error_code: null - ok, 1 - ei lubata päringut teha, tagasatada veateade; 2 - sooritada kehtivuspäring kontrollregistrist
CREATE OR REPLACE FUNCTION misp2.check_portal(
	portal_short_name text, 
	represented_party_class text, 
	represented_party_code text) RETURNS misp2.ret_type1 AS $BODY$
DECLARE
    portal_row misp2.portal%ROWTYPE;
    org_row misp2.org%ROWTYPE;
    unit_org_row misp2.org%ROWTYPE;
    org_valid_row misp2.org_valid%ROWTYPE;
    check_register_status_row misp2.check_register_status%ROWTYPE;
    ret misp2.ret_type1;
BEGIN
 	SELECT INTO 
		portal_row.id, portal_row.misp_type, portal_row.univ_check_query, portal_row.org_id, portal_row.security_host, 
			portal_row.xroad_protocol_ver, portal_row.xroad_instance, org_row.id, org_row.member_class, org_row.code, org_row.subsystem_code
			
        po.id, po.misp_type, po.univ_check_query, po.org_id, po.security_host, 
			po.xroad_protocol_ver, po.xroad_instance, org.id, org.member_class, org.code, org.subsystem_code
			
  		FROM misp2.portal po, misp2.org org
 		WHERE po.org_id = org.id AND po.short_name=portal_short_name;

  	ret.portal_id := portal_row.id;
  	ret.client_xroad_instance :=  portal_row.xroad_instance;
  	ret.client_member_class :=  org_row.member_class;
  	ret.client_member_code :=  org_row.code;
  	ret.client_subsystem_code :=  org_row.subsystem_code;

	IF NOT FOUND THEN
	    ret.error_message := 'Antud nimega portaal puudub (' || portal_short_name ||')!';
	    ret.error_code := 1;
	    return ret;
	END IF;
  
	IF portal_row.misp_type > 1 THEN 
		IF represented_party_code IS  NULL OR char_length(represented_party_code) = 0 THEN
			ret.error_message := 'header/allasutus (unit) peab olema antud!';
			ret.error_code := 1;
			return ret;
		END IF;
		IF portal_row.xroad_protocol_ver = '4.0' THEN
			SELECT INTO unit_org_row *
			FROM misp2.org
			WHERE member_class = represented_party_class AND code = represented_party_code;
			IF NOT FOUND THEN
				ret.error_message := 'Üksus puudub portaalis (' || represented_party_class || ':' || represented_party_code || ')!';
				ret.error_code := 1;
				return ret;
			END IF;
		ELSE
			SELECT INTO unit_org_row *
			FROM misp2.org
			WHERE code = represented_party_code;
			
			IF NOT FOUND THEN
				ret.error_message := 'Üksus puudub portaalis (' || represented_party_code ||')!';
				ret.error_code := 1;
				return ret;
			END IF;
		END IF;

		IF portal_row.univ_check_query IS NOT NULL AND portal_row.univ_check_valid_time IS NOT NULL AND portal_row.univ_check_valid_time > 0 THEN
			SELECT INTO org_valid_row *
			FROM misp2.org_valid
			WHERE org_id = unit_org_row.id;
			IF NOT FOUND THEN
				--do_check := true;
			ELSIF org_valid_row.valid_date + portal_row.univ_check_valid_time * INTERVAL '1 hour' < NOW() THEN
				ret.error_message := 'Org is valid';
				ret.error_code := NULL; 
				return ret;
			END IF;

			SELECT INTO check_register_status_row *
			FROM misp2.check_register_status
			WHERE query_name = portal_row.univ_check_query;
			IF NOT FOUND THEN
				ret.error_message := 'check_register_status not found';
				ret.error_code := 2;
				return ret;
			END IF;
			IF check_register_status_row.is_ok = 1 THEN
				ret.error_message := 'check_register_status ok';
				ret.error_code := 2;
				return ret;
			ELSIF check_register_status_row.last_modified + 5 * INTERVAL '1 minute' < NOW() THEN
				ret.error_message := 'check_register_status not ok, wait for check interval';
				ret.error_code := NULL; 
				return ret;
			ELSE
				ret.error_message := 'check_register_status not ok, check interval passed';
				ret.error_code := 2;
				return ret;			
			END IF;
		END IF;
	END IF;
	ret.error_message := 'ok';
	ret.error_code := NULL; 
	return ret;
END;
$BODY$LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION misp2.check_portal(text, text, text) OWNER TO misp2;


-- DROP FUNCTION misp2.is_allowed(text, text, text, text);
DROP TYPE IF EXISTS misp2.ret_type2 CASCADE;
CREATE TYPE misp2.ret_type2 AS (sec_host varchar(128), error_code varchar(128), error_message varchar(512));

DROP FUNCTION IF EXISTS misp2.is_allowed(text, text, text, integer);
--user_id, service_name, unit, portal_id
CREATE OR REPLACE FUNCTION misp2.is_allowed(
	user_ssn text, 
	service_xroad_instance text, 
	service_member_class text,
	service_member_code text,
	service_subsystem_code text,
	service_name text,
	represented_party_class text,
	represented_party_code text,
	portal_id integer) RETURNS misp2.ret_type2 AS $BODY$
DECLARE
    portal_row misp2.portal%ROWTYPE;
    ret misp2.ret_type2;
    current_producer_id integer;
    current_unit_id integer;
    test_id integer;
BEGIN

	SELECT INTO portal_row * FROM misp2.portal WHERE id=portal_id;

	IF NOT FOUND THEN
		ret.sec_host := '-1';
		ret.error_message := 'Portaal puudub!';
		ret.error_code := NULL;
		return ret;
	END IF;

	IF portal_row.xroad_protocol_ver = '4.0' THEN
		-- x-road instances must be the same, do not send if they are different
		IF portal_row.xroad_instance = service_xroad_instance THEN
			-- find current_producer_id for given portal and service
			-- either with or without subsystem code
			IF service_subsystem_code IS NULL OR char_length(service_subsystem_code) = 0 THEN
				SELECT id INTO current_producer_id FROM misp2.producer WHERE 
					producer.portal_id = portal_row.id AND 
					producer.member_class = service_member_class AND
					producer.short_name = service_member_code AND
					(producer.subsystem_code IS NULL OR producer.subsystem_code = '');
			ELSE
				SELECT id INTO current_producer_id FROM misp2.producer WHERE 
					producer.portal_id = portal_row.id AND 
					producer.member_class = service_member_class AND
					producer.short_name = service_member_code AND
					producer.subsystem_code = service_subsystem_code;
					
				--RAISE NOTICE  'SELECT * FROM misp2.producer WHERE  producer.portal_id = ''%'' AND producer.member_class = ''%'' AND producer.short_name = ''%'' AND producer.subsystem_code = ''%'' ', portal_row.id  , service_member_class , service_member_code , service_subsystem_code;
			END IF;
			 
			IF current_producer_id IS NULL THEN
				ret.error_message := 'header/service ei ole korrektne!';
			ELSE
				IF portal_row.misp_type > 1 THEN
					-- find current_unit_id (misp2.org.id) for given portal and unit represent (with member class filter)
					SELECT id INTO current_unit_id FROM misp2.org WHERE
							org.sup_org_id = portal_row.org_id AND
							org.member_class = represented_party_class AND
							org.code = represented_party_code;
					IF current_unit_id IS NULL THEN
						ret.error_message := 'header/allasutus (representedParty) ei ole korrektne!';
					END IF;
				END IF;
			END IF;
		ELSE
			ret.error_message := 'header/xRoadInstance kliendi ja teenuse x-tee instantsid on erinevad(' || portal_row.xroad_instance || '/' || service_xroad_instance || ')!';
		END IF;
		
	ELSE
		-- find producer id for given portal and service
		SELECT id INTO current_producer_id FROM misp2.producer WHERE 
			producer.portal_id = portal_row.id AND 
			producer.short_name = service_member_code;
	
		IF current_producer_id IS NULL THEN
			ret.error_message := 'header/nimi (service) ei ole korrektne!';
		ELSE
			IF portal_row.misp_type > 1 THEN
				-- find current_unit_id (misp2.org.id) for given portal and unit represent (no member class filter)
				SELECT id INTO current_unit_id FROM misp2.org WHERE 
						org.sup_org_id = portal_row.org_id AND
						org.code = represented_party_code;
			
				IF current_unit_id IS NULL THEN
					ret.error_message := 'header/allasutus (unit) ei ole korrektne!';
				END IF;
			END IF;
		END IF;
	END IF;
	-- throw error if error message is set
	IF ret.error_message IS NOT NULL AND char_length(ret.error_message) > 0 THEN
	   	ret.sec_host := '-1';
	   	ret.error_code := NULL;
	   	return ret;
	END IF;
  
	-- Here we are certain that the service and unit exist, now need to verify if user has access rights to use this service
    IF portal_row.misp_type = 0 THEN
      	SELECT INTO test_id gi.id
      		FROM misp2.group_item gi
        		INNER JOIN misp2.org_query oq ON (gi.org_query_id = oq.id)
        		INNER JOIN misp2.query q ON (oq.query_id=q.id AND q.name = service_name)
				INNER JOIN misp2.producer pr ON (q.producer_id = pr.id AND pr.id = current_producer_id);
    ELSIF portal_row.misp_type = 1 THEN
      	SELECT INTO test_id gi.id
     		FROM misp2.person p
        		INNER JOIN misp2.org_person op ON (p.id = op.person_id AND (op.role & 13) > 0)
        		INNER JOIN misp2.org o ON (op.org_id = o.id AND o.id = portal_row.org_id)
        		INNER JOIN misp2.group_person gp ON (p.id = gp.person_id AND gp.org_id = o.id)
        		INNER JOIN misp2.group_ g ON (gp.group_id = g.id AND g.org_id = o.id)
        		INNER JOIN misp2.group_item gi ON (g.id = gi.group_id)
        		INNER JOIN misp2.org_query oq ON (gi.org_query_id = oq.id)
        		INNER JOIN misp2.query q ON (oq.query_id = q.id AND q.name = service_name)
        		INNER JOIN misp2.producer pr ON (q.producer_id = pr.id AND pr.id = current_producer_id)
    		WHERE p.ssn = user_ssn;
    ELSIF portal_row.misp_type = 2 OR portal_row.misp_type = 3 THEN
      	SELECT INTO test_id gi.id
      		FROM misp2.person p
        		INNER JOIN misp2.org_person op ON (p.id = op.person_id AND (op.role & 13) > 0)
        		INNER JOIN misp2.org o ON (op.org_id = o.id AND o.id = current_unit_id)
        		INNER JOIN misp2.portal p0 ON (portal_row.org_id=o.id OR portal_row.org_id=o.sup_org_id)
        		INNER JOIN misp2.group_person gp ON (p.id = gp.person_id AND gp.org_id = o.id)
        		INNER JOIN misp2.group_ g ON (gp.group_id = g.id AND (g.org_id = o.id OR g.org_id = o.sup_org_id) and g.portal_id=portal_row.id)
        		INNER JOIN misp2.group_item gi ON (g.id = gi.group_id)
        		INNER JOIN misp2.org_query oq ON (gi.org_query_id = oq.id)
        		INNER JOIN misp2.query q ON (oq.query_id = q.id AND q.name = service_name)
        		INNER JOIN misp2.producer pr ON (q.producer_id = pr.id AND pr.id = current_producer_id)
      		WHERE p.ssn = user_ssn;
    END IF;

    IF NOT FOUND THEN
       ret.sec_host := '-1';
       ret.error_message := 'Puudub päringu sooritamise õigus!';
       ret.error_code := NULL;
       return ret;
    ELSE
       	ret.sec_host := portal_row.security_host || '/cgi-bin/consumer_proxy';
       	ret.error_message := NULL;
       	ret.error_code := NULL;
       	return ret;
    END IF;
  	return ret;
END;
$BODY$LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION misp2.is_allowed(text, text, text, text, text, text, text, text, integer) OWNER TO misp2;
COMMENT ON FUNCTION misp2.is_allowed(text, text, text, text, text, text, text, text, integer) IS 'Checks whether user has the right to execute the query or not.';


